
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>

#include "cgdk.h"
#include "dpmi.h"

#define DOS_INT 0x21
#define DOS_GETLISTOFLISTS 0x52

struct TMCB
{
  char type;
  unsigned short owner;
  unsigned short size;
  char reserved [3];
  char name [8];
};

static TMCB *MCB_getFirst (void)
{
  RMINFO rmi;
  short *listOfLists;

  memset (&rmi, 0, sizeof rmi);
  rmi.h.ah = DOS_GETLISTOFLISTS;
  DPMI_int (DOS_INT, &rmi);
  listOfLists = (short *) MK_DP (rmi.x.es,rmi.x.ebx);
  listOfLists--;

  return (TMCB *) MK_DP (*listOfLists, 0);
}

static TMCB *MCB_getNext (TMCB *mcb)
{
  if (mcb->type == 'M')
  {
    return (TMCB *) MK_DP ((DP_SEG (mcb) + 1 + mcb->size),0);
  }
  else
  {
    return NULL;
  }
}

TMCB **MCB_createList (void)
{
  TMCB **list;
  TMCB *mcb;
  int i;
  int size = sizeof (TMCB *);

  list = (TMCB * *) _alloc (size);
  if (list == NULL)
    return NULL;
  mcb = MCB_getFirst ();
  i = 0;
  do
  {
    if ((mcb->owner-1) == DP_SEG (mcb))
    {
      list [i] = mcb;
      size += sizeof (TMCB *);
      i++;
      list = (TMCB * *) _realloc (list, size,NULL);
      if (list == NULL)
      {
        return NULL;
      }
    }
    mcb = MCB_getNext (mcb);
  }
  while (mcb != NULL);
  list [i] = NULL;

  return list;
}

static void MCB_freeList (TMCB **list)
{
  if (list != NULL)
  {
    _free (list);
  }
}

static char **MCB_createNames (void)
{
  TMCB **list;
  char **clist;
  int len = 0;

  list = MCB_createList ();
  if (list != NULL)
  {
    while (list [len] != NULL)
      len++;
    clist = (char * *) _calloc (sizeof (char *)+1, len,NULL);
    if (clist == NULL)
    {
      MCB_freeList (list);
      return NULL;
    }
    len = 0;
    while (list [len] != NULL)
    {
      clist [len] = (char *) _alloc (strlen ( (list [len]->name))+1);
      if (clist [len])
      {
        strcpy (clist [len], list [len]->name);
      }
      if (strlen (clist [len]) > 8)
        clist [len] [8] = 0;
      len++;
    }
    clist [len] = NULL;
    MCB_freeList (list);
  }
  else
  {
    return NULL;
  }

  return clist;
}

static void MCB_freeNames (char **names)
{
  int i;

  if (names)
  {
    for (i = 0; names [i] != NULL; i++)
    {
      _free (names [i]);
    }
    _free (names);
  }
}

